@@ -30,9 +30,10 @@ states, visit :ref:`the Audit State machine <audit_state_machine>`.
 """
 
 import datetime
-from dateutil import tz
+from datetime import timezone
 
 from http import HTTPStatus
+import jsonschema
 from oslo_log import log
 from oslo_utils import timeutils
 import pecan
@@ -76,6 +77,8 @@ def hide_fields_in_newer_versions(obj):
         obj.end_time = wtypes.Unset
     if not api_utils.allow_force():
         obj.force = wtypes.Unset
+    if not api_utils.allow_skipped_action():
+        obj.status_message = wtypes.Unset
 
 
 class AuditPostType(wtypes.Base):
@@ -114,6 +117,11 @@ class AuditPostType(wtypes.Base):
         if self.audit_type not in audit_type_values:
             raise exception.AuditTypeNotFound(audit_type=self.audit_type)
 
+        if not self.audit_template_uuid and not self.goal:
+            message = _(
+                'A valid goal or audit_template_id must be provided')
+            raise exception.Invalid(message)
+
         if (self.audit_type == objects.audit.AuditType.ONESHOT.value and
                 self.interval not in (wtypes.Unset, None)):
             raise exception.AuditIntervalNotAllowed(audit_type=self.audit_type)
@@ -207,6 +215,14 @@ class AuditPatchType(types.JsonPatchType):
     def mandatory_attrs():
         return ['/audit_template_uuid', '/type']
 
+    @staticmethod
+    def internal_attrs():
+        # There are global internal attributes and object specific ones.
+        # /status_message is only modified internally based on state changes
+        # and is not exposed in the patch API for Audits.
+        audit_internal_attrs = ['/status_message']
+        return types.JsonPatchType.internal_attrs() + audit_internal_attrs
+
     @staticmethod
     def validate(patch):
 
@@ -369,6 +385,9 @@ class Audit(base.APIBase):
     """Allow Action Plan of this Audit be executed in parallel
        with other Action Plan"""
 
+    status_message = wtypes.wsattr(wtypes.text, mandatory=False)
+    """Status message of the audit"""
+
     def __init__(self, **kwargs):
         self.fields = []
         fields = list(objects.Audit.fields)
@@ -473,10 +492,6 @@ class AuditsController(rest.RestController):
         super(AuditsController, self).__init__()
         self.dc_client = rpcapi.DecisionEngineAPI()
 
-    from_audits = False
-    """A flag to indicate if the requests to this controller are coming
-    from the top-level resource Audits."""
-
     _custom_actions = {
         'detail': ['GET'],
     }
@@ -588,9 +603,6 @@ class AuditsController(rest.RestController):
 
         :param audit: UUID or name of an audit.
         """
-        if self.from_audits:
-            raise exception.OperationNotPermitted
-
         context = pecan.request.context
         rpc_audit = api_utils.get_resource('Audit', audit)
         policy.enforce(context, 'audit:get', rpc_audit, action='audit:get')
@@ -609,14 +621,6 @@ class AuditsController(rest.RestController):
                        action='audit:create')
         audit = audit_p.as_audit(context)
 
-        if self.from_audits:
-            raise exception.OperationNotPermitted
-
-        if not audit._goal_uuid:
-            raise exception.Invalid(
-                message=_('A valid goal_id or audit_template_id '
-                          'must be provided'))
-
         strategy_uuid = audit.strategy_uuid
         no_schema = True
         if strategy_uuid is not None:
@@ -627,8 +631,12 @@ class AuditsController(rest.RestController):
             if schema:
                 # validate input parameter with default value feedback
                 no_schema = False
-                utils.StrictDefaultValidatingDraft4Validator(schema).validate(
-                    audit.parameters)
+                try:
+                    utils.StrictDefaultValidatingDraft4Validator(
+                        schema).validate(audit.parameters)
+                except jsonschema.exceptions.ValidationError as e:
+                    raise exception.Invalid(
+                        _('Invalid parameters for strategy: %s') % e)
 
         if no_schema and audit.parameters:
             raise exception.Invalid(_('Specify parameters but no predefined '
@@ -640,13 +648,11 @@ class AuditsController(rest.RestController):
         start_time_value = audit_dict.get('start_time')
         end_time_value = audit_dict.get('end_time')
         if start_time_value:
-            audit_dict['start_time'] = start_time_value.replace(
-                tzinfo=tz.tzlocal()).astimezone(
-                    tz.tzutc()).replace(tzinfo=None)
+            audit_dict['start_time'] = start_time_value.astimezone(
+                timezone.utc).replace(tzinfo=None)
         if end_time_value:
-            audit_dict['end_time'] = end_time_value.replace(
-                tzinfo=tz.tzlocal()).astimezone(
-                    tz.tzutc()).replace(tzinfo=None)
+            audit_dict['end_time'] = end_time_value.astimezone(
+                timezone.utc).replace(tzinfo=None)
 
         new_audit = objects.Audit(context, **audit_dict)
         new_audit.create()
@@ -668,9 +674,6 @@ class AuditsController(rest.RestController):
         :param audit: UUID or name of an audit.
         :param patch: a json PATCH document to apply to this audit.
         """
-        if self.from_audits:
-            raise exception.OperationNotPermitted
-
         context = pecan.request.context
         audit_to_update = api_utils.get_resource(
             'Audit', audit, eager=True)
@@ -696,9 +699,8 @@ class AuditsController(rest.RestController):
                 patch_value = api_utils.get_patch_value(patch, patch_path)
                 # convert string format to UTC time
                 new_patch_value = wutils.parse_isodatetime(
-                    patch_value).replace(
-                        tzinfo=tz.tzlocal()).astimezone(
-                            tz.tzutc()).replace(tzinfo=None)
+                    patch_value).astimezone(
+                        timezone.utc).replace(tzinfo=None)
                 api_utils.set_patch_value(patch, patch_path, new_patch_value)
 
             audit = Audit(**api_utils.apply_jsonpatch(audit_dict, patch))
