@@ -23,6 +23,7 @@ import pecan
 import wsme
 
 from watcher._i18n import _
+from watcher.api.controllers.v1 import versions
 from watcher.common import utils
 from watcher import objects
 
@@ -101,6 +102,18 @@ def get_patch_value(patch, key):
             return p['value']
 
 
+def set_patch_value(patch, key, value):
+    for p in patch:
+        if p['op'] == 'replace' and p['path'] == '/%s' % key:
+            p['value'] = value
+
+
+def get_patch_key(patch, key):
+    for p in patch:
+        if p['op'] == 'replace' and key in p.keys():
+            return p[key][1:]
+
+
 def check_audit_state_transition(patch, initial):
     is_transition_valid = True
     state_value = get_patch_value(patch, "state")
@@ -143,3 +156,60 @@ def get_resource(resource, resource_id, eager=False):
         return _get(pecan.request.context, resource_id, eager=eager)
 
     return _get(pecan.request.context, resource_id)
+
+
+def allow_start_end_audit_time():
+    """Check if we should support optional start/end attributes for Audit.
+
+    Version 1.1 of the API added support for start and end time of continuous
+    audits.
+    """
+    return pecan.request.version.minor >= (
+        versions.VERSIONS.MINOR_1_START_END_TIMING.value)
+
+
+def allow_force():
+    """Check if we should support optional force attribute for Audit.
+
+    Version 1.2 of the API added support for forced audits that allows to
+    launch audit when other action plan is ongoing.
+    """
+    return pecan.request.version.minor >= (
+        versions.VERSIONS.MINOR_2_FORCE.value)
+
+
+def allow_list_datamodel():
+    """Check if we should support list data model API.
+
+    Version 1.3 of the API added support to list data model.
+    """
+    return pecan.request.version.minor >= (
+        versions.VERSIONS.MINOR_3_DATAMODEL.value)
+
+
+def allow_webhook_api():
+    """Check if we should support webhook API.
+
+    Version 1.4 of the API added support to trigger webhook.
+    """
+    return pecan.request.version.minor >= (
+        versions.VERSIONS.MINOR_4_WEBHOOK_API.value)
+
+
+def allow_skipped_action():
+    """Check if we should support skipped action.
+
+    Version 1.5 of the API added support to skipped actions.
+    """
+    return pecan.request.version.minor >= (
+        versions.VERSIONS.MINOR_5_SKIPPED_ACTION.value)
+
+
+def allow_list_extend_compute_model():
+    """Check if it should list extended compute model attributes.
+
+    Version 1.6 of the API added support to additional attributes
+    to the compute data model.
+    """
+    return pecan.request.version.minor >= (
+        versions.VERSIONS.MINOR_6_EXT_COMPUTE_MODEL.value)
