@@ -20,20 +20,21 @@
 
 import sys
 
-from oslo_log import log as logging
+from oslo_log import log
 
-from watcher._i18n import _LI
-from watcher.common import service as service
+from watcher.common import service
+from watcher import conf
 from watcher.decision_engine import sync
 
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
+CONF = conf.CONF
 
 
 def main():
-    LOG.info(_LI('Watcher sync started.'))
+    LOG.info('Watcher sync started.')
 
-    service.prepare_service(sys.argv)
+    service.prepare_service(sys.argv, CONF)
     syncer = sync.Syncer()
     syncer.sync()
 
-    LOG.info(_LI('Watcher sync finished.'))
+    LOG.info('Watcher sync finished.')
