@@ -17,7 +17,8 @@ from oslo_config import cfg
 from oslo_log import log
 import oslo_messaging as messaging
 
-from watcher._i18n import _LE
+from oslo_messaging.rpc import dispatcher
+
 from watcher.common import context as watcher_context
 from watcher.common import exception
 
@@ -32,7 +33,6 @@ __all__ = [
     'get_client',
     'get_server',
     'get_notifier',
-    'TRANSPORT_ALIASES',
 ]
 
 CONF = cfg.CONF
@@ -46,16 +46,6 @@ ALLOWED_EXMODS = [
 ]
 EXTRA_EXMODS = []
 
-# NOTE(lucasagomes): The watcher.openstack.common.rpc entries are for
-# backwards compat with IceHouse rpc_backend configuration values.
-TRANSPORT_ALIASES = {
-    'watcher.openstack.common.rpc.impl_kombu': 'rabbit',
-    'watcher.openstack.common.rpc.impl_qpid': 'qpid',
-    'watcher.openstack.common.rpc.impl_zmq': 'zmq',
-    'watcher.rpc.impl_kombu': 'rabbit',
-    'watcher.rpc.impl_qpid': 'qpid',
-    'watcher.rpc.impl_zmq': 'zmq',
-}
 
 JsonPayloadSerializer = messaging.JsonPayloadSerializer
 
@@ -63,17 +53,18 @@ JsonPayloadSerializer = messaging.JsonPayloadSerializer
 def init(conf):
     global TRANSPORT, NOTIFICATION_TRANSPORT, NOTIFIER
     exmods = get_allowed_exmods()
-    TRANSPORT = messaging.get_transport(conf,
-                                        allowed_remote_exmods=exmods,
-                                        aliases=TRANSPORT_ALIASES)
+    TRANSPORT = messaging.get_rpc_transport(
+        conf, allowed_remote_exmods=exmods)
     NOTIFICATION_TRANSPORT = messaging.get_notification_transport(
-        conf,
-        allowed_remote_exmods=exmods,
-        aliases=TRANSPORT_ALIASES)
+        conf, allowed_remote_exmods=exmods)
 
     serializer = RequestContextSerializer(JsonPayloadSerializer())
-    NOTIFIER = messaging.Notifier(NOTIFICATION_TRANSPORT,
-                                  serializer=serializer)
+    if not conf.notification_level:
+        NOTIFIER = messaging.Notifier(
+            NOTIFICATION_TRANSPORT, serializer=serializer, driver='noop')
+    else:
+        NOTIFIER = messaging.Notifier(NOTIFICATION_TRANSPORT,
+                                      serializer=serializer)
 
 
 def initialized():
@@ -83,7 +74,7 @@ def initialized():
 def cleanup():
     global TRANSPORT, NOTIFICATION_TRANSPORT, NOTIFIER
     if NOTIFIER is None:
-        LOG.exception(_LE("RPC cleanup: NOTIFIER is None"))
+        LOG.exception("RPC cleanup: NOTIFIER is None")
     TRANSPORT.cleanup()
     NOTIFICATION_TRANSPORT.cleanup()
     TRANSPORT = NOTIFICATION_TRANSPORT = NOTIFIER = None
@@ -121,7 +112,7 @@ class RequestContextSerializer(messaging.Serializer):
         return self._base.deserialize_entity(context, entity)
 
     def serialize_context(self, context):
-        return context
+        return context.to_dict()
 
     def deserialize_context(self, context):
         return watcher_context.RequestContext.from_dict(context)
@@ -130,24 +121,40 @@ class RequestContextSerializer(messaging.Serializer):
 def get_client(target, version_cap=None, serializer=None):
     assert TRANSPORT is not None
     serializer = RequestContextSerializer(serializer)
-    return messaging.RPCClient(TRANSPORT,
-                               target,
-                               version_cap=version_cap,
-                               serializer=serializer)
+    return messaging.get_rpc_client(
+        TRANSPORT,
+        target,
+        version_cap=version_cap,
+        serializer=serializer
+    )
 
 
 def get_server(target, endpoints, serializer=None):
     assert TRANSPORT is not None
+    access_policy = dispatcher.DefaultRPCAccessPolicy
     serializer = RequestContextSerializer(serializer)
-    return messaging.get_rpc_server(TRANSPORT,
-                                    target,
-                                    endpoints,
-                                    executor='eventlet',
-                                    serializer=serializer)
+    return messaging.get_rpc_server(
+        TRANSPORT,
+        target,
+        endpoints,
+        serializer=serializer,
+        access_policy=access_policy
+    )
 
 
-def get_notifier(service=None, host=None, publisher_id=None):
+def get_notification_listener(targets, endpoints, serializer=None, pool=None):
+    assert NOTIFICATION_TRANSPORT is not None
+    serializer = RequestContextSerializer(serializer)
+    return messaging.get_notification_listener(
+        NOTIFICATION_TRANSPORT,
+        targets,
+        endpoints,
+        allow_requeue=False,
+        pool=pool,
+        serializer=serializer
+    )
+
+
+def get_notifier(publisher_id):
     assert NOTIFIER is not None
-    if not publisher_id:
-        publisher_id = "%s.%s" % (service, host or CONF.host)
     return NOTIFIER.prepare(publisher_id=publisher_id)
