@@ -15,8 +15,6 @@
 # limitations under the License.
 #
 
-from __future__ import print_function
-
 import collections
 import datetime
 import itertools
@@ -25,9 +23,8 @@ import sys
 from oslo_log import log
 from oslo_utils import strutils
 import prettytable as ptable
-from six.moves import input
 
-from watcher._i18n import _, _LI
+from watcher._i18n import _
 from watcher._i18n import lazy_translation_enabled
 from watcher.common import context
 from watcher.common import exception
@@ -145,27 +142,27 @@ class PurgeCommand(object):
         return expiry_date
 
     @classmethod
-    def get_audit_template_uuid(cls, uuid_or_name):
+    def get_goal_uuid(cls, uuid_or_name):
         if uuid_or_name is None:
             return
 
         query_func = None
         if not utils.is_uuid_like(uuid_or_name):
-            query_func = objects.AuditTemplate.get_by_name
+            query_func = objects.Goal.get_by_name
         else:
-            query_func = objects.AuditTemplate.get_by_uuid
+            query_func = objects.Goal.get_by_uuid
 
         try:
-            audit_template = query_func(cls.ctx, uuid_or_name)
+            goal = query_func(cls.ctx, uuid_or_name)
         except Exception as exc:
             LOG.exception(exc)
-            raise exception.AuditTemplateNotFound(audit_template=uuid_or_name)
+            raise exception.GoalNotFound(goal=uuid_or_name)
 
-        if not audit_template.deleted_at:
+        if not goal.deleted_at:
             raise exception.NotSoftDeletedStateError(
-                name=_('Audit Template'), id=uuid_or_name)
+                name=_('Goal'), id=uuid_or_name)
 
-        return audit_template.uuid
+        return goal.uuid
 
     def _find_goals(self, filters=None):
         return objects.Goal.list(self.ctx, filters=filters)
@@ -209,18 +206,19 @@ class PurgeCommand(object):
             (audit_template.strategy_id and
              audit_template.strategy_id not in strategy_ids)]
 
-        audit_template_ids = [at.id for at in audit_templates
-                              if at not in orphans.audit_templates]
         orphans.audits = [
             audit for audit in audits
-            if audit.audit_template_id not in audit_template_ids]
+            if audit.goal_id not in goal_ids or
+            (audit.strategy_id and
+             audit.strategy_id not in strategy_ids)]
 
         # Objects with orphan parents are themselves orphans
         audit_ids = [audit.id for audit in audits
                      if audit not in orphans.audits]
         orphans.action_plans = [
             ap for ap in action_plans
-            if ap.audit_id not in audit_ids]
+            if ap.audit_id not in audit_ids or
+            ap.strategy_id not in strategy_ids]
 
         # Objects with orphan parents are themselves orphans
         action_plan_ids = [ap.id for ap in action_plans
@@ -230,7 +228,7 @@ class PurgeCommand(object):
             if action.action_plan_id not in action_plan_ids]
 
         LOG.debug("Orphans found:\n%s", orphans)
-        LOG.info(_LI("Orphans found:\n%s"), orphans.get_count_table())
+        LOG.info("Orphans found:\n%s", orphans.get_count_table())
 
         return orphans
 
@@ -270,6 +268,7 @@ class PurgeCommand(object):
             related_objs = WatcherObjectsMap()
             related_objs.strategies = self._find_strategies(filters)
             related_objs.audit_templates = self._find_audit_templates(filters)
+            related_objs.audits = self._find_audits(filters)
             objects_map += related_objs
 
         for strategy in objects_map.strategies:
@@ -278,13 +277,6 @@ class PurgeCommand(object):
             filters.update(dict(strategy_id=strategy.id))
             related_objs = WatcherObjectsMap()
             related_objs.audit_templates = self._find_audit_templates(filters)
-            objects_map += related_objs
-
-        for audit_template in objects_map.audit_templates:
-            filters = {}
-            filters.update(base_filters)
-            filters.update(dict(audit_template_id=audit_template.id))
-            related_objs = WatcherObjectsMap()
             related_objs.audits = self._find_audits(filters)
             objects_map += related_objs
 
@@ -355,12 +347,9 @@ class PurgeCommand(object):
             ]
 
             # audits
-            audit_template_ids = [
-                audit_template.id
-                for audit_template in related_objs.audit_templates]
             related_objs.audits = [
                 audit for audit in self._objects_map.audits
-                if audit.audit_template_id in audit_template_ids
+                if audit.goal_id in goal_ids
             ]
 
             # action plans
@@ -411,13 +400,13 @@ class PurgeCommand(object):
         return to_be_deleted
 
     def do_delete(self):
-        LOG.info(_LI("Deleting..."))
+        LOG.info("Deleting...")
         # Reversed to avoid errors with foreign keys
         for entry in reversed(list(self._objects_map)):
             entry.destroy()
 
     def execute(self):
-        LOG.info(_LI("Starting purge command"))
+        LOG.info("Starting purge command")
         self._objects_map = self.find_objects_to_delete()
 
         if (self.max_number is not None and
@@ -432,7 +421,7 @@ class PurgeCommand(object):
         if not self.dry_run and self.confirmation_prompt():
             self.do_delete()
             print(_("Purge results summary%s:") % _orphans_note)
-            LOG.info(_LI("Purge results summary%s:"), _orphans_note)
+            LOG.info("Purge results summary%s:", _orphans_note)
         else:
             LOG.debug(self._objects_map)
             print(_("Here below is a table containing the objects "
@@ -440,10 +429,10 @@ class PurgeCommand(object):
 
         LOG.info("\n%s", self._objects_map.get_count_table())
         print(self._objects_map.get_count_table())
-        LOG.info(_LI("Purge process completed"))
+        LOG.info("Purge process completed")
 
 
-def purge(age_in_days, max_number, audit_template, exclude_orphans, dry_run):
+def purge(age_in_days, max_number, goal, exclude_orphans, dry_run):
     """Removes soft deleted objects from the database
 
     :param age_in_days: Number of days since deletion (from today)
@@ -452,8 +441,8 @@ def purge(age_in_days, max_number, audit_template, exclude_orphans, dry_run):
     :param max_number: Max number of objects expected to be deleted.
                   Prevents the deletion if exceeded. No limit if set to None.
     :type max_number: int
-    :param audit_template: UUID or name of the audit template to purge.
-    :type audit_template: str
+    :param goal: UUID or name of the goal to purge.
+    :type goal: str
     :param exclude_orphans: Flag to indicate whether or not you want to
                             exclude orphans from deletion (default: False).
     :type exclude_orphans: bool
@@ -467,11 +456,11 @@ def purge(age_in_days, max_number, audit_template, exclude_orphans, dry_run):
 
         LOG.info("[options] age_in_days = %s", age_in_days)
         LOG.info("[options] max_number = %s", max_number)
-        LOG.info("[options] audit_template = %s", audit_template)
+        LOG.info("[options] goal = %s", goal)
         LOG.info("[options] exclude_orphans = %s", exclude_orphans)
         LOG.info("[options] dry_run = %s", dry_run)
 
-        uuid = PurgeCommand.get_audit_template_uuid(audit_template)
+        uuid = PurgeCommand.get_goal_uuid(goal)
 
         cmd = PurgeCommand(age_in_days, max_number, uuid,
                            exclude_orphans, dry_run)
