@@ -30,6 +30,7 @@ class InstanceState(enum.Enum):
     RESCUED = 'rescued'  # A rescue image is running with the original image
     # attached.
     RESIZED = 'resized'  # an Instance with the new size is active.
+    SHELVED = 'shelved'
 
     SOFT_DELETED = 'soft-delete'
     # still available to restore.
@@ -45,14 +46,17 @@ class Instance(compute_resource.ComputeResource):
         # If the resource is excluded by the scope,
         # 'watcher_exclude' property will be set True.
         "watcher_exclude": wfields.BooleanField(default=False),
+        "name": wfields.StringField(),
         "state": wfields.StringField(default=InstanceState.ACTIVE.value),
-
         "memory": wfields.NonNegativeIntegerField(),
-        "disk": wfields.IntegerField(),
-        "disk_capacity": wfields.NonNegativeIntegerField(),
+        "disk": wfields.NonNegativeIntegerField(),
         "vcpus": wfields.NonNegativeIntegerField(),
         "metadata": wfields.JsonField(),
         "project_id": wfields.UUIDField(),
+        "locked": wfields.BooleanField(default=False),
+        # New fields for extended compute model
+        "pinned_az": wfields.StringField(default=""),
+        "flavor_extra_specs": wfields.JsonField(default={}),
     }
 
     def accept(self, visitor):
