@@ -18,11 +18,13 @@
 # limitations under the License.
 #
 
-from oslo_config import cfg
+import collections
+
 from oslo_log import log
-import six
+import oslo_utils
 
 from watcher._i18n import _
+from watcher.applier.actions import migration
 from watcher.common import exception
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
@@ -49,7 +51,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
     A capacity coefficients (cc) might be used to adjust optimization
     thresholds. Different resources may require different coefficient
     values as well as setting up different coefficient values in both
-    phases may lead to to more efficient consolidation in the end.
+    phases may lead to more efficient consolidation in the end.
     If the cc equals 1 the full resource capacity may be used, cc
     values lower than 1 will lead to resource under utilization and
     values higher than 1 will lead to resource overbooking.
@@ -66,36 +68,24 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
     an active compute node to any other active compute node.
     """
 
-    HOST_CPU_USAGE_METRIC_NAME = 'compute.node.cpu.percent'
-    INSTANCE_CPU_USAGE_METRIC_NAME = 'cpu_util'
-
+    AGGREGATE = 'mean'
     DATASOURCE_METRICS = ['instance_ram_allocated', 'instance_cpu_usage',
-                          'instance_ram_usage', 'instance_root_disk_size']
-
-    METRIC_NAMES = dict(
-        ceilometer=dict(
-            cpu_util_metric='cpu_util',
-            ram_util_metric='memory.resident',
-            ram_alloc_metric='memory',
-            disk_alloc_metric='disk.root.size'),
-        gnocchi=dict(
-            cpu_util_metric='cpu_util',
-            ram_util_metric='memory.resident',
-            ram_alloc_metric='memory',
-            disk_alloc_metric='disk.root.size'),
-    )
+                          'instance_ram_usage', 'instance_root_disk_size',
+                          'host_cpu_usage', 'host_ram_usage']
 
     MIGRATION = "migrate"
     CHANGE_NOVA_SERVICE_STATE = "change_nova_service_state"
 
     def __init__(self, config, osc=None):
         super(VMWorkloadConsolidation, self).__init__(config, osc)
-        self._ceilometer = None
-        self._gnocchi = None
         self.number_of_migrations = 0
         self.number_of_released_nodes = 0
-        # self.ceilometer_instance_data_cache = dict()
         self.datasource_instance_data_cache = dict()
+        self.datasource_node_data_cache = dict()
+        # Host metric adjustments that take into account planned
+        # migrations.
+        self.host_metric_delta = collections.defaultdict(
+            lambda: collections.defaultdict(int))
 
     @classmethod
     def get_name(cls):
@@ -137,21 +127,12 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
             }
         }
 
-    @classmethod
-    def get_config_opts(cls):
-        return [
-            cfg.StrOpt(
-                "datasource",
-                help="Data source to use in order to query the needed metrics",
-                default="gnocchi",
-                choices=["ceilometer", "gnocchi"])
-        ]
-
     def get_available_compute_nodes(self):
         default_node_scope = [element.ServiceState.ENABLED.value,
                               element.ServiceState.DISABLED.value]
+        nodes = self.compute_model.get_all_compute_nodes().items()
         return {uuid: cn for uuid, cn in
-                self.compute_model.get_all_compute_nodes().items()
+                nodes
                 if cn.state == element.ServiceState.ONLINE.value and
                 cn.status in default_node_scope}
 
@@ -160,7 +141,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
 
         :param instance:
         """
-        if isinstance(instance.state, six.string_types):
+        if isinstance(instance.state, str):
             return instance.state
         elif isinstance(instance.state, element.InstanceState):
             return instance.state.value
@@ -176,7 +157,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
 
         :param node:
         """
-        if isinstance(node.status, six.string_types):
+        if isinstance(node.status, str):
             return node.status
         elif isinstance(node.status, element.ServiceState):
             return node.status.value
@@ -193,7 +174,8 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         :param node: node object
         :return: None
         """
-        params = {'state': element.ServiceState.ENABLED.value}
+        params = {'state': element.ServiceState.ENABLED.value,
+                  'resource_name': node.hostname}
         self.solution.add_action(
             action_type=self.CHANGE_NOVA_SERVICE_STATE,
             resource_id=node.uuid,
@@ -207,7 +189,8 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         :return: None
         """
         params = {'state': element.ServiceState.DISABLED.value,
-                  'disabled_reason': self.REASON_FOR_DISABLE}
+                  'disabled_reason': self.REASON_FOR_DISABLE,
+                  'resource_name': node.hostname}
         self.solution.add_action(
             action_type=self.CHANGE_NOVA_SERVICE_STATE,
             resource_id=node.uuid,
@@ -223,12 +206,12 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         :return: None
         """
         instance_state_str = self.get_instance_state_str(instance)
-        if instance_state_str not in (element.InstanceState.ACTIVE.value,
-                                      element.InstanceState.PAUSED.value):
-            # Watcher currently only supports live VM migration and block live
-            # VM migration which both requires migrated VM to be active.
-            # When supported, the cold migration may be used as a fallback
-            # migration mechanism to move non active VMs.
+        if instance_state_str in (element.InstanceState.ACTIVE.value,
+                                  element.InstanceState.PAUSED.value):
+            migration_type = migration.Migrate.LIVE_MIGRATION
+        elif instance_state_str == element.InstanceState.STOPPED.value:
+            migration_type = migration.Migrate.COLD_MIGRATION
+        else:
             LOG.error(
                 'Cannot live migrate: instance_uuid=%(instance_uuid)s, '
                 'state=%(instance_state)s.', dict(
@@ -236,8 +219,6 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
                     instance_state=instance_state_str))
             return
 
-        migration_type = 'live'
-
         # Here will makes repeated actions to enable the same compute node,
         # when migrating VMs to the destination node which is disabled.
         # Whether should we remove the same actions in the solution???
@@ -248,14 +229,25 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
 
         if self.compute_model.migrate_instance(
                 instance, source_node, destination_node):
-            params = {'migration_type': migration_type,
-                      'source_node': source_node.uuid,
-                      'destination_node': destination_node.uuid}
-            self.solution.add_action(action_type=self.MIGRATION,
-                                     resource_id=instance.uuid,
-                                     input_parameters=params)
+            self.add_action_migrate(
+                instance,
+                migration_type,
+                source_node,
+                destination_node)
             self.number_of_migrations += 1
 
+            instance_util = self.get_instance_utilization(instance)
+            self.host_metric_delta[source_node.hostname]['cpu'] -= (
+                instance_util['cpu'])
+            # We'll deduce the vm allocated memory.
+            self.host_metric_delta[source_node.hostname]['ram'] -= (
+                instance.memory)
+
+            self.host_metric_delta[destination_node.hostname]['cpu'] += (
+                instance_util['cpu'])
+            self.host_metric_delta[destination_node.hostname]['ram'] += (
+                instance.memory)
+
     def disable_unused_nodes(self):
         """Generate actions for disabling unused nodes.
 
@@ -281,36 +273,21 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         if instance.uuid in self.datasource_instance_data_cache.keys():
             return self.datasource_instance_data_cache.get(instance.uuid)
 
-        cpu_util_metric = self.METRIC_NAMES[
-            self.config.datasource]['cpu_util_metric']
-        ram_util_metric = self.METRIC_NAMES[
-            self.config.datasource]['ram_util_metric']
-        ram_alloc_metric = self.METRIC_NAMES[
-            self.config.datasource]['ram_alloc_metric']
-        disk_alloc_metric = self.METRIC_NAMES[
-            self.config.datasource]['disk_alloc_metric']
-
-        instance_cpu_util = self.datasource_backend.statistic_aggregation(
-            resource_id=instance.uuid,
-            meter_name=cpu_util_metric,
-            period=self.period,
-            granularity=self.granularity)
-        instance_ram_util = self.datasource_backend.statistic_aggregation(
-            resource_id=instance.uuid,
-            meter_name=ram_util_metric,
-            period=self.period,
-            granularity=self.granularity)
+        instance_cpu_util = self.datasource_backend.get_instance_cpu_usage(
+            resource=instance, period=self.period,
+            aggregate=self.AGGREGATE, granularity=self.granularity)
+        instance_ram_util = self.datasource_backend.get_instance_ram_usage(
+            resource=instance, period=self.period,
+            aggregate=self.AGGREGATE, granularity=self.granularity)
         if not instance_ram_util:
-            instance_ram_util = self.datasource_backend.statistic_aggregation(
-                resource_id=instance.uuid,
-                meter_name=ram_alloc_metric,
-                period=self.period,
-                granularity=self.granularity)
-        instance_disk_util = self.datasource_backend.statistic_aggregation(
-            resource_id=instance.uuid,
-            meter_name=disk_alloc_metric,
-            period=self.period,
-            granularity=self.granularity)
+            instance_ram_util = (
+                self.datasource_backend.get_instance_ram_allocated(
+                    resource=instance, period=self.period,
+                    aggregate=self.AGGREGATE, granularity=self.granularity))
+        instance_disk_util = (
+            self.datasource_backend.get_instance_root_disk_size(
+                resource=instance, period=self.period,
+                aggregate=self.AGGREGATE, granularity=self.granularity))
 
         if instance_cpu_util:
             total_cpu_utilization = (
@@ -333,6 +310,21 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
             disk=instance_disk_util)
         return self.datasource_instance_data_cache.get(instance.uuid)
 
+    def _get_node_total_utilization(self, node):
+        if node.hostname in self.datasource_node_data_cache:
+            return self.datasource_node_data_cache[node.hostname]
+
+        cpu = self.datasource_backend.get_host_cpu_usage(
+            node, self.period, self.AGGREGATE,
+            self.granularity)
+        ram = self.datasource_backend.get_host_ram_usage(
+            node, self.period, self.AGGREGATE,
+            self.granularity)
+
+        self.datasource_node_data_cache[node.hostname] = dict(
+            cpu=cpu, ram=ram)
+        return self.datasource_node_data_cache[node.hostname]
+
     def get_node_utilization(self, node):
         """Collect cpu, ram and disk utilization statistics of a node.
 
@@ -350,8 +342,36 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
             node_cpu_util += instance_util['cpu']
             node_ram_util += instance_util['ram']
             node_disk_util += instance_util['disk']
+            LOG.debug("instance utilization: %s %s",
+                      instance, instance_util)
 
-        return dict(cpu=node_cpu_util, ram=node_ram_util,
+        total_node_util = self._get_node_total_utilization(node)
+        total_node_cpu_util = total_node_util['cpu'] or 0
+        if total_node_cpu_util:
+            total_node_cpu_util = total_node_cpu_util * node.vcpus / 100
+            # account for planned migrations
+            total_node_cpu_util += self.host_metric_delta[node.hostname]['cpu']
+
+        total_node_ram_util = total_node_util['ram'] or 0
+        if total_node_ram_util:
+            total_node_ram_util /= oslo_utils.units.Ki
+            total_node_ram_util += self.host_metric_delta[node.hostname]['ram']
+
+        LOG.debug(
+            "node utilization: %s. "
+            "total instance cpu: %s, "
+            "total instance ram: %s, "
+            "total instance disk: %s, "
+            "total host cpu: %s, "
+            "total host ram: %s, "
+            "node delta usage: %s.",
+            node,
+            node_cpu_util, node_ram_util, node_disk_util,
+            total_node_cpu_util, total_node_ram_util,
+            self.host_metric_delta[node.hostname])
+
+        return dict(cpu=max(node_cpu_util, total_node_cpu_util),
+                    ram=max(node_ram_util, total_node_ram_util),
                     disk=node_disk_util)
 
     def get_node_capacity(self, node):
@@ -360,7 +380,8 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         :param node: node object
         :return: dict(cpu(cores), ram(MB), disk(B))
         """
-        return dict(cpu=node.vcpus, ram=node.memory, disk=node.disk_capacity)
+        return dict(cpu=node.vcpu_capacity, ram=node.memory_mb_capacity,
+                    disk=node.disk_gb_capacity)
 
     def get_relative_node_utilization(self, node):
         """Return relative node utilization.
@@ -430,8 +451,15 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         instance_utilization = self.get_instance_utilization(instance)
         metrics = ['cpu', 'ram', 'disk']
         for m in metrics:
-            if (instance_utilization[m] + node_utilization[m] >
-                    node_capacity[m] * cc[m]):
+            fits = (instance_utilization[m] + node_utilization[m] <=
+                    node_capacity[m] * cc[m])
+            LOG.debug(
+                "Instance fits: %s, metric: %s, instance: %s, "
+                "node: %s, instance utilization: %s, "
+                "node utilization: %s, node capacity: %s, cc: %s",
+                fits, m, instance, node, instance_utilization[m],
+                node_utilization[m], node_capacity[m], cc[m])
+            if not fits:
                 return False
         return True
 
@@ -461,13 +489,16 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
                     'input_parameters'][
                         'resource_id'] == instance_uuid)
             if len(actions) > 1:
-                src_uuid = actions[0]['input_parameters']['source_node']
-                dst_uuid = actions[-1]['input_parameters']['destination_node']
+                src_name = actions[0]['input_parameters']['source_node']
+                dst_name = actions[-1]['input_parameters']['destination_node']
                 for a in actions:
                     self.solution.actions.remove(a)
                     self.number_of_migrations -= 1
-                src_node = self.compute_model.get_node_by_uuid(src_uuid)
-                dst_node = self.compute_model.get_node_by_uuid(dst_uuid)
+                LOG.info("Optimized migrations: %s. "
+                         "Source: %s, destination: %s", actions,
+                         src_name, dst_name)
+                src_node = self.compute_model.get_node_by_name(src_name)
+                dst_node = self.compute_model.get_node_by_name(dst_name)
                 instance = self.compute_model.get_instance_by_uuid(
                     instance_uuid)
                 if self.compute_model.migrate_instance(
@@ -502,6 +533,8 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
                         key=lambda x: self.get_instance_utilization(
                             x)['cpu']
                 ):
+                    LOG.info("Node %s overloaded, attempting to reduce load.",
+                             node)
                     # skip exclude instance when migrating
                     if instance.watcher_exclude:
                         LOG.debug("Instance is excluded by scope, "
@@ -510,11 +543,19 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
                     for destination_node in reversed(sorted_nodes):
                         if self.instance_fits(
                                 instance, destination_node, cc):
+                            LOG.info("Offload: found fitting "
+                                     "destination (%s) for instance: %s. "
+                                     "Planning migration.",
+                                     destination_node, instance.uuid)
                             self.add_migration(instance, node,
                                                destination_node)
                             break
                     if not self.is_overloaded(node, cc):
+                        LOG.info("Node %s no longer overloaded.", node)
                         break
+                    else:
+                        LOG.info("Node still overloaded (%s), "
+                                 "continuing offload phase.", node)
 
     def consolidation_phase(self, cc):
         """Perform consolidation phase.
@@ -550,6 +591,10 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
                         break
                     if self.instance_fits(
                             instance, destination_node, cc):
+                        LOG.info("Consolidation: found fitting "
+                                 "destination (%s) for instance: %s. "
+                                 "Planning migration.",
+                                 destination_node, instance.uuid)
                         self.add_migration(instance, node,
                                            destination_node)
                         break
@@ -557,15 +602,9 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
             asc += 1
 
     def pre_execute(self):
-        if not self.compute_model:
-            raise exception.ClusterStateNotDefined()
+        self._pre_execute()
 
-        if self.compute_model.stale:
-            raise exception.ClusterStateStale()
-
-        LOG.debug(self.compute_model.to_string())
-
-    def do_execute(self):
+    def do_execute(self, audit=None):
         """Execute strategy.
 
         This strategy produces a solution resulting in more
