@@ -12,9 +12,8 @@
 # limitations under the License.
 #
 
-from dateutil.parser import parse
-
 from oslo_log import log
+from oslo_utils import timeutils
 
 from cinderclient.v3.volumes import Volume
 from novaclient.v2.servers import Server
@@ -413,6 +412,13 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
             LOG.debug("%s %s", dst_pool, dst_type)
 
             if src_type == dst_type:
+                if dst_pool is None:
+                    LOG.warning("volume %s will not be migrated because "
+                                "it already has type %s and 'dst_pool'"
+                                " was not specified for src_pool %s",
+                                volume.name, dst_type,
+                                pool)
+                    continue
                 self._volume_migrate(volume, dst_pool)
             else:
                 self._volume_retype(volume, dst_type)
@@ -449,9 +455,12 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
 
     def _live_migration(self, instance, src_node, dst_node):
         parameters = {"migration_type": "live",
-                      "destination_node": dst_node,
                       "source_node": src_node,
                       "resource_name": instance.name}
+        if dst_node:
+            # if dst_node is None, do not add it to the parameters for the
+            # migration action, and let Nova figure out the destination node
+            parameters["destination_node"] = dst_node
         self.solution.add_action(
             action_type="migrate",
             resource_id=instance.id,
@@ -460,9 +469,12 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
 
     def _cold_migration(self, instance, src_node, dst_node):
         parameters = {"migration_type": "cold",
-                      "destination_node": dst_node,
                       "source_node": src_node,
                       "resource_name": instance.name}
+        if dst_node:
+            # if dst_node is None, do not add it to the parameters for the
+            # migration action, and let Nova figure out the destination node
+            parameters["destination_node"] = dst_node
         self.solution.add_action(
             action_type="migrate",
             resource_id=instance.id,
@@ -558,7 +570,7 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
         filter_actions = self.get_priority_filter_list()
         LOG.debug(filter_actions)
 
-        # apply all filters set in input prameter
+        # apply all filters set in input parameter
         for action in list(reversed(filter_actions)):
             LOG.debug(action)
             result = action.apply_filter(result)
@@ -790,7 +802,7 @@ class ComputeHostSortFilter(SortMovingToFrontFilter):
 
         :param item: instance object
         :param sort_key: compute host name
-        :returns: true: compute name on which intance is equals sort_key
+        :returns: true: compute name on where instance host equals sort_key
                   false: otherwise
         """
 
@@ -818,7 +830,7 @@ class StorageHostSortFilter(SortMovingToFrontFilter):
 
         :param item: volume object
         :param sort_key: storage pool name
-        :returns: true: pool name on which intance is equals sort_key
+        :returns: true: pool name on where instance.host equals sort_key
                   false: otherwise
         """
 
@@ -882,7 +894,8 @@ class ComputeSpecSortFilter(BaseFilter):
                             reverse=True)
         elif sort_key == 'created_at':
             result = sorted(items,
-                            key=lambda x: parse(getattr(x, sort_key)),
+                            key=lambda x: timeutils.parse_isotime(
+                                getattr(x, sort_key)),
                             reverse=False)
 
         return result
@@ -948,7 +961,8 @@ class StorageSpecSortFilter(BaseFilter):
 
         if sort_key == 'created_at':
             result = sorted(items,
-                            key=lambda x: parse(getattr(x, sort_key)),
+                            key=lambda x: timeutils.parse_isotime(
+                                getattr(x, sort_key)),
                             reverse=False)
         else:
             result = sorted(items,
