@@ -22,20 +22,25 @@ def is_patched():
 
 
 def _monkey_patch():
-    if is_patched():
-        return
     # Anything imported here will not be monkey patched. It is
     # important to take care not to import anything here which requires monkey
     # patching. eventlet processes environment variables at import-time.
     # as such any eventlet configuration should happen here if needed.
     import eventlet
     eventlet.monkey_patch()
+    global MONKEY_PATCHED
+    MONKEY_PATCHED = True
+
+
+def _is_patching_enabled():
+    if (os.environ.get('OS_WATCHER_DISABLE_EVENTLET_PATCHING', '').lower()
+            not in ('1', 'true', 'yes', 'y')):
+        return True
+    return False
 
 
 def patch():
-    # This is only for debugging, this should not be used in production.
-    if (os.environ.get('OS_WATCHER_DISABLE_EVENTLET_PATCHING', '').lower()
-            not in ('1', 'true', 'yes', 'y')):
+    # NOTE(dviroel): monkey_patch when called, even if is already patched.
+    # Ignore if the control flag is disabling patching
+    if _is_patching_enabled():
         _monkey_patch()
-        global MONKEY_PATCHED
-        MONKEY_PATCHED = True
