@@ -23,12 +23,12 @@
 # NOTE(deva): import auth_token so we can override a config option
 
 import copy
-import mock
+from unittest import mock
+from urllib import parse as urlparse
 
 from oslo_config import cfg
 import pecan
 import pecan.testing
-from six.moves.urllib import parse as urlparse
 
 from watcher.api import hooks
 from watcher.common import context as watcher_context
@@ -79,7 +79,6 @@ class FunctionalTest(base.DbTestCase):
                     hooks.ContextHook(),
                     hooks.NoExceptionTracebackHook()
                 ],
-                'static_root': '%s/public' % root_dir,
                 'template_path': '%s/api/templates' % root_dir,
                 'enable_acl': enable_acl,
                 'acl_public_routes': ['/', '/v1'],
@@ -106,7 +105,6 @@ class FunctionalTest(base.DbTestCase):
         :param path_prefix: prefix of the url path
         """
         full_path = path_prefix + path
-        print('%s: %s %s' % (method.upper(), full_path, params))
 
         response = getattr(self.app, "%s_json" % method)(
             str(full_path),
@@ -116,7 +114,6 @@ class FunctionalTest(base.DbTestCase):
             extra_environ=extra_environ,
             expect_errors=expect_errors
         )
-        print('GOT:%s' % response)
         return response
 
     def put_json(self, path, params, expect_errors=False, headers=None,
@@ -193,17 +190,16 @@ class FunctionalTest(base.DbTestCase):
         :param path_prefix: prefix of the url path
         """
         full_path = path_prefix + path
-        print('DELETE: %s' % (full_path))
         response = self.app.delete(str(full_path),
                                    headers=headers,
                                    status=status,
                                    extra_environ=extra_environ,
                                    expect_errors=expect_errors)
-        print('GOT:%s' % response)
         return response
 
     def get_json(self, path, expect_errors=False, headers=None,
-                 extra_environ=None, q=[], path_prefix=PATH_PREFIX, **params):
+                 extra_environ=None, q=[], path_prefix=PATH_PREFIX,
+                 return_json=True, **params):
         """Sends simulated HTTP GET request to Pecan test app.
 
         :param path: url path of target service
@@ -229,16 +225,14 @@ class FunctionalTest(base.DbTestCase):
         all_params.update(params)
         if q:
             all_params.update(query_params)
-        print('GET: %s %r' % (full_path, all_params))
 
         response = self.app.get(full_path,
                                 params=all_params,
                                 headers=headers,
                                 extra_environ=extra_environ,
                                 expect_errors=expect_errors)
-        if not expect_errors:
+        if return_json and not expect_errors:
             response = response.json
-        print('GOT:%s' % response)
         return response
 
     def validate_link(self, link, bookmark=False):
