@@ -10,9 +10,8 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import datetime
+import fixtures
 import itertools
-from unittest import mock
 
 from http import HTTPStatus
 from oslo_config import cfg
@@ -68,13 +67,43 @@ class TestListAction(api_base.FunctionalTest):
         for field in action_fields:
             self.assertIn(field, action)
 
-    def test_one(self):
+    def test_list(self):
         action = obj_utils.create_test_action(self.context, parents=None)
         response = self.get_json('/actions')
         self.assertEqual(action.uuid, response['actions'][0]["uuid"])
         self._assert_action_fields(response['actions'][0])
+        self.assertNotIn('status_message', response['actions'][0])
 
-    def test_one_soft_deleted(self):
+    def test_list_with_status_message(self):
+        action = obj_utils.create_test_action(
+            self.context, parents=None, status_message='Fake message')
+        response = self.get_json(
+            '/actions', headers={'OpenStack-API-Version': 'infra-optim 1.5'})
+        self.assertEqual(action.uuid, response['actions'][0]["uuid"])
+        self._assert_action_fields(response['actions'][0])
+        # status_message is not in the basic actions list
+        self.assertNotIn('status_message', response['actions'][0])
+
+    def test_list_detail_with_hidden_status_message(self):
+        action = obj_utils.create_test_action(
+            self.context, status_message='Fake message', parents=None)
+        response = self.get_json('/actions/detail')
+        self.assertEqual(action.uuid, response['actions'][0]["uuid"])
+        self._assert_action_fields(response['actions'][0])
+        self.assertNotIn('status_message', response['actions'][0])
+
+    def test_list_detail_with_status_message(self):
+        action = obj_utils.create_test_action(
+            self.context, status_message='Fake message', parents=None)
+        response = self.get_json(
+            '/actions/detail',
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'})
+        self.assertEqual(action.uuid, response['actions'][0]["uuid"])
+        self._assert_action_fields(response['actions'][0])
+        self.assertEqual(
+            'Fake message', response['actions'][0]["status_message"])
+
+    def test_list_soft_deleted(self):
         action = obj_utils.create_test_action(self.context, parents=None)
         action.soft_delete()
         response = self.get_json('/actions',
@@ -85,15 +114,51 @@ class TestListAction(api_base.FunctionalTest):
         response = self.get_json('/actions')
         self.assertEqual([], response['actions'])
 
-    def test_get_one(self):
+    def test_show(self):
         action = obj_utils.create_test_action(self.context, parents=None)
         response = self.get_json('/actions/%s' % action['uuid'])
         self.assertEqual(action.uuid, response['uuid'])
         self.assertEqual(action.action_type, response['action_type'])
         self.assertEqual(action.input_parameters, response['input_parameters'])
+        self.assertNotIn('status_message', response)
         self._assert_action_fields(response)
 
-    def test_get_one_soft_deleted(self):
+    def test_show_with_status_message(self):
+        action = obj_utils.create_test_action(
+            self.context, parents=None, status_message='test')
+        response = self.get_json(
+            '/actions/%s' % action['uuid'],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'})
+        self.assertEqual(action.uuid, response['uuid'])
+        self.assertEqual(action.action_type, response['action_type'])
+        self.assertEqual(action.input_parameters, response['input_parameters'])
+        self.assertEqual('test', response['status_message'])
+        self._assert_action_fields(response)
+
+    def test_show_with_hidden_status_message(self):
+        action = obj_utils.create_test_action(
+            self.context, parents=None, status_message='test')
+        response = self.get_json(
+            '/actions/%s' % action['uuid'],
+            headers={'OpenStack-API-Version': 'infra-optim 1.4'})
+        self.assertEqual(action.uuid, response['uuid'])
+        self.assertEqual(action.action_type, response['action_type'])
+        self.assertEqual(action.input_parameters, response['input_parameters'])
+        self.assertNotIn('status_message', response)
+        self._assert_action_fields(response)
+
+    def test_show_with_empty_status_message(self):
+        action = obj_utils.create_test_action(self.context, parents=None)
+        response = self.get_json(
+            '/actions/%s' % action['uuid'],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'})
+        self.assertEqual(action.uuid, response['uuid'])
+        self.assertEqual(action.action_type, response['action_type'])
+        self.assertEqual(action.input_parameters, response['input_parameters'])
+        self.assertIsNone(response['status_message'])
+        self._assert_action_fields(response)
+
+    def test_show_soft_deleted(self):
         action = obj_utils.create_test_action(self.context, parents=None)
         action.soft_delete()
         response = self.get_json('/actions/%s' % action['uuid'],
@@ -105,13 +170,13 @@ class TestListAction(api_base.FunctionalTest):
                                  expect_errors=True)
         self.assertEqual(HTTPStatus.NOT_FOUND, response.status_int)
 
-    def test_detail(self):
+    def test_list_detail(self):
         action = obj_utils.create_test_action(self.context, parents=None)
         response = self.get_json('/actions/detail')
         self.assertEqual(action.uuid, response['actions'][0]["uuid"])
         self._assert_action_fields(response['actions'][0])
 
-    def test_detail_soft_deleted(self):
+    def test_list_detail_soft_deleted(self):
         action = obj_utils.create_test_action(self.context, parents=None)
         action.soft_delete()
         response = self.get_json('/actions/detail',
@@ -122,13 +187,13 @@ class TestListAction(api_base.FunctionalTest):
         response = self.get_json('/actions/detail')
         self.assertEqual([], response['actions'])
 
-    def test_detail_against_single(self):
+    def test_show_detail(self):
         action = obj_utils.create_test_action(self.context, parents=None)
         response = self.get_json('/actions/%s/detail' % action['uuid'],
                                  expect_errors=True)
         self.assertEqual(HTTPStatus.NOT_FOUND, response.status_int)
 
-    def test_many(self):
+    def test_list_multiple_actions(self):
         action_list = []
         for id_ in range(5):
             action = obj_utils.create_test_action(self.context, id=id_,
@@ -139,7 +204,7 @@ class TestListAction(api_base.FunctionalTest):
         uuids = [s['uuid'] for s in response['actions']]
         self.assertEqual(sorted(action_list), sorted(uuids))
 
-    def test_many_with_action_plan_uuid(self):
+    def test_list_with_action_plan_uuid(self):
         action_plan = obj_utils.create_test_action_plan(
             self.context,
             id=2,
@@ -157,7 +222,7 @@ class TestListAction(api_base.FunctionalTest):
         for action in response['actions']:
             self.assertEqual(action_plan.uuid, action['action_plan_uuid'])
 
-    def test_filter_by_audit_uuid(self):
+    def test_list_filter_by_audit_uuid(self):
         action_plan_1 = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid())
@@ -189,7 +254,7 @@ class TestListAction(api_base.FunctionalTest):
         for action in response['actions']:
             self.assertEqual(action_plan_1.uuid, action['action_plan_uuid'])
 
-    def test_filter_by_action_plan_uuid(self):
+    def test_list_filter_by_action_plan_uuid(self):
         action_plan_1 = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
@@ -225,7 +290,7 @@ class TestListAction(api_base.FunctionalTest):
         for action in response['actions']:
             self.assertEqual(action_plan_2.uuid, action['action_plan_uuid'])
 
-    def test_details_and_filter_by_action_plan_uuid(self):
+    def test_list_details_and_filter_by_action_plan_uuid(self):
         action_plan = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
@@ -242,7 +307,7 @@ class TestListAction(api_base.FunctionalTest):
         for action in response['actions']:
             self.assertEqual(action_plan.uuid, action['action_plan_uuid'])
 
-    def test_details_and_filter_by_audit_uuid(self):
+    def test_list_details_and_filter_by_audit_uuid(self):
         action_plan = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
@@ -259,7 +324,7 @@ class TestListAction(api_base.FunctionalTest):
         for action in response['actions']:
             self.assertEqual(action_plan.uuid, action['action_plan_uuid'])
 
-    def test_filter_by_action_plan_and_audit_uuids(self):
+    def test_list_filter_by_action_plan_and_audit_uuids(self):
         action_plan = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
@@ -269,7 +334,7 @@ class TestListAction(api_base.FunctionalTest):
         response = self.get_json(url, expect_errors=True)
         self.assertEqual(HTTPStatus.BAD_REQUEST, response.status_int)
 
-    def test_many_with_sort_key_uuid(self):
+    def test_list_with_sort_key_uuid(self):
         action_plan = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
@@ -290,7 +355,7 @@ class TestListAction(api_base.FunctionalTest):
             sorted([a.uuid for a in actions_list]),
             names)
 
-    def test_many_with_sort_key_action_plan_uuid(self):
+    def test_list_with_sort_key_action_plan_uuid(self):
         action_plan_1 = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
@@ -324,13 +389,13 @@ class TestListAction(api_base.FunctionalTest):
                 action_plan_uuids,
                 message='Failed on %s direction' % direction)
 
-    def test_sort_key_validation(self):
+    def test_list_sort_key_validation(self):
         response = self.get_json(
             '/actions?sort_key=%s' % 'bad_name',
             expect_errors=True)
         self.assertEqual(HTTPStatus.BAD_REQUEST, response.status_int)
 
-    def test_many_with_soft_deleted_action_plan_uuid(self):
+    def test_list_with_soft_deleted_action_plan_uuid(self):
         action_plan1 = obj_utils.create_test_action_plan(
             self.context,
             id=2,
@@ -378,7 +443,7 @@ class TestListAction(api_base.FunctionalTest):
             set([act['uuid'] for act in response['actions']
                  if act['action_plan_uuid'] == action_plan2.uuid]))
 
-    def test_many_with_parents(self):
+    def test_list_with_parents(self):
         action_list = []
         for id_ in range(5):
             if id_ > 0:
@@ -396,7 +461,7 @@ class TestListAction(api_base.FunctionalTest):
             self.assertEqual(response_actions[id_]['uuid'],
                              response_actions[id_ + 1]['parents'][0])
 
-    def test_many_without_soft_deleted(self):
+    def test_list_without_soft_deleted(self):
         action_list = []
         for id_ in [1, 2, 3]:
             action = obj_utils.create_test_action(self.context, id=id_,
@@ -411,7 +476,7 @@ class TestListAction(api_base.FunctionalTest):
         uuids = [s['uuid'] for s in response['actions']]
         self.assertEqual(sorted(action_list), sorted(uuids))
 
-    def test_many_with_soft_deleted(self):
+    def test_list_with_soft_deleted(self):
         action_list = []
         for id_ in [1, 2, 3]:
             action = obj_utils.create_test_action(self.context, id=id_,
@@ -428,7 +493,7 @@ class TestListAction(api_base.FunctionalTest):
         uuids = [s['uuid'] for s in response['actions']]
         self.assertEqual(sorted(action_list), sorted(uuids))
 
-    def test_links(self):
+    def test_show_with_links(self):
         uuid = utils.generate_uuid()
         obj_utils.create_test_action(self.context, id=1, uuid=uuid)
         response = self.get_json('/actions/%s' % uuid)
@@ -440,7 +505,7 @@ class TestListAction(api_base.FunctionalTest):
             self.assertTrue(self.validate_link(
                 link['href'], bookmark=bookmark))
 
-    def test_collection_links(self):
+    def test_list_with_limit(self):
         parents = None
         for id_ in range(5):
             action = obj_utils.create_test_action(self.context, id=id_,
@@ -450,7 +515,7 @@ class TestListAction(api_base.FunctionalTest):
         response = self.get_json('/actions/?limit=3')
         self.assertEqual(3, len(response['actions']))
 
-    def test_collection_links_default_limit(self):
+    def test_list_with_default_max_limit(self):
         cfg.CONF.set_override('max_limit', 3, 'api')
         for id_ in range(5):
             obj_utils.create_test_action(self.context, id=id_,
@@ -459,68 +524,192 @@ class TestListAction(api_base.FunctionalTest):
         self.assertEqual(3, len(response['actions']))
 
 
-class TestPatch(api_base.FunctionalTest):
+class TestPatchAction(api_base.FunctionalTest):
 
     def setUp(self):
-        super(TestPatch, self).setUp()
-        obj_utils.create_test_goal(self.context)
-        obj_utils.create_test_strategy(self.context)
-        obj_utils.create_test_audit(self.context)
-        obj_utils.create_test_action_plan(self.context)
-        self.action = obj_utils.create_test_action(self.context, parents=None)
-        p = mock.patch.object(db_api.BaseConnection, 'update_action')
-        self.mock_action_update = p.start()
-        self.mock_action_update.side_effect = self._simulate_rpc_action_update
-        self.addCleanup(p.stop)
-
-    def _simulate_rpc_action_update(self, action):
-        action.save()
-        return action
-
-    @mock.patch('oslo_utils.timeutils.utcnow')
-    def test_patch_not_allowed(self, mock_utcnow):
-        test_time = datetime.datetime(2000, 1, 1, 0, 0)
-        mock_utcnow.return_value = test_time
-        new_state = objects.audit.State.SUCCEEDED
-        response = self.get_json('/actions/%s' % self.action.uuid)
-        self.assertNotEqual(new_state, response['state'])
-
-        response = self.patch_json(
-            '/actions/%s' % self.action.uuid,
-            [{'path': '/state', 'value': new_state, 'op': 'replace'}],
-            expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(HTTPStatus.FORBIDDEN, response.status_int)
-        self.assertTrue(response.json['error_message'])
-
-
-class TestDelete(api_base.FunctionalTest):
-
-    def setUp(self):
-        super(TestDelete, self).setUp()
+        super(TestPatchAction, self).setUp()
         self.goal = obj_utils.create_test_goal(self.context)
         self.strategy = obj_utils.create_test_strategy(self.context)
         self.audit = obj_utils.create_test_audit(self.context)
-        self.action_plan = obj_utils.create_test_action_plan(self.context)
+        self.action_plan = obj_utils.create_test_action_plan(
+            self.context,
+            state=objects.action_plan.State.PENDING)
         self.action = obj_utils.create_test_action(self.context, parents=None)
-        p = mock.patch.object(db_api.BaseConnection, 'update_action')
-        self.mock_action_update = p.start()
-        self.mock_action_update.side_effect = self._simulate_rpc_action_update
-        self.addCleanup(p.stop)
+        self.mock_action_update = self.useFixture(
+            fixtures.MockPatchObject(
+                db_api.BaseConnection, "update_action",
+                autospec=False,
+                side_effect=self._simulate_rpc_action_update)
+        ).mock.return_value
 
     def _simulate_rpc_action_update(self, action):
         action.save()
         return action
 
-    @mock.patch('oslo_utils.timeutils.utcnow')
-    def test_delete_action_not_allowed(self, mock_utcnow):
-        test_time = datetime.datetime(2000, 1, 1, 0, 0)
-        mock_utcnow.return_value = test_time
-        response = self.delete('/actions/%s' % self.action.uuid,
-                               expect_errors=True)
-        self.assertEqual(HTTPStatus.FORBIDDEN, response.status_int)
+    def test_patch_action_not_allowed_old_microversion(self):
+        """Test that action patch is not allowed in older microversions"""
+        new_state = objects.action.State.SKIPPED
+        response = self.get_json('/actions/%s' % self.action.uuid)
+        self.assertNotEqual(new_state, response['state'])
+
+        # Test with API version 1.4 (should fail)
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/state', 'value': new_state, 'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.4'},
+            expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, response.status_int)
+        self.assertTrue(response.json['error_message'])
+
+    def test_patch_action_allowed_new_microversion(self):
+        """Test that action patch is allowed in microversion 1.5+"""
+        new_state = objects.action.State.SKIPPED
+        response = self.get_json('/actions/%s' % self.action.uuid)
+        self.assertNotEqual(new_state, response['state'])
+
+        # Test with API version 1.5 (should succeed)
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/state', 'value': new_state, 'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'})
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(HTTPStatus.OK, response.status_int)
+        self.assertEqual(new_state, response.json['state'])
+        self.assertEqual('Action skipped by user.',
+                         response.json['status_message'])
+
+    def test_patch_action_invalid_state_transition(self):
+        """Test that invalid state transitions are rejected"""
+        # Try to transition from PENDING to SUCCEEDED (should fail)
+        new_state = objects.action.State.SUCCEEDED
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/state', 'value': new_state, 'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'},
+            expect_errors=True)
+        self.assertEqual(HTTPStatus.CONFLICT, response.status_int)
         self.assertEqual('application/json', response.content_type)
         self.assertTrue(response.json['error_message'])
+        self.assertIn("State transition not allowed: (PENDING -> SUCCEEDED)",
+                      response.json['error_message'])
+
+    def test_patch_action_skip_non_pending_ap(self):
+        """Test transition conditions on parent actionplan
+
+        The PENDING to SKIPPED transition is not allowed if
+        the actionplan is not PENDING or RECOMMENDED state
+        """
+        self.action_plan.state = objects.action_plan.State.ONGOING
+        self.action_plan.save()
+        new_state = objects.action.State.SKIPPED
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/state', 'value': new_state, 'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'},
+            expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(HTTPStatus.CONFLICT, response.status_int)
+        self.assertTrue(response.json['error_message'])
+        self.assertIn("State update not allowed for actionplan state: ONGOING",
+                      response.json['error_message'])
+
+    def test_patch_action_skip_transition_with_status_message(self):
+        """Test that PENDING to SKIPPED transition is allowed"""
+        new_state = objects.action.State.SKIPPED
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/state', 'value': new_state, 'op': 'replace'},
+             {'path': '/status_message', 'value': 'test message',
+              'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'})
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(HTTPStatus.OK, response.status_int)
+        self.assertEqual(new_state, response.json['state'])
+        self.assertEqual(
+            'Action skipped by user. Reason: test message',
+            response.json['status_message'])
+
+    def test_patch_action_invalid_state_value(self):
+        """Test that invalid state values are rejected"""
+        invalid_state = "INVALID_STATE"
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/state', 'value': invalid_state, 'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'},
+            expect_errors=True)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['error_message'])
+
+    def test_patch_action_remove_status_message_not_allowed(self):
+        """Test that remove fields is not allowed"""
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/status_message', 'op': 'remove'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'},
+            expect_errors=True)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['error_message'])
+        self.assertIn("is a mandatory attribute and can not be removed",
+                      response.json['error_message'])
+
+    def test_patch_action_status_message_not_allowed(self):
+        """Test status_message cannot be patched directly when not SKIPPED"""
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/status_message', 'value': 'test message',
+              'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'},
+            expect_errors=True)
+        self.assertEqual(HTTPStatus.CONFLICT, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertIn("status_message update only allowed when action state "
+                      "is SKIPPED", response.json['error_message'])
+        self.assertIsNone(self.action.status_message)
+
+    def test_patch_action_one_allowed_one_not_allowed(self):
+        """Test that status_message cannot be patched directly"""
+        new_state = objects.action.State.SKIPPED
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/state', 'value': new_state, 'op': 'replace'},
+             {'path': '/action_plan_id', 'value': 56, 'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'},
+            expect_errors=True)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['error_message'])
+        self.assertIn("\'/action_plan_id\' is not an allowed attribute and "
+                      "can not be updated", response.json['error_message'])
+        self.assertIsNone(self.action.status_message)
+
+    def test_patch_action_status_message_allowed_when_skipped(self):
+        """Test that status_message can be updated when action is SKIPPED"""
+        # First transition to SKIPPED state
+        new_state = objects.action.State.SKIPPED
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/state', 'value': new_state, 'op': 'replace'},
+             {'path': '/status_message', 'value': 'initial message',
+              'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'})
+        self.assertEqual(HTTPStatus.OK, response.status_int)
+        self.assertEqual(new_state, response.json['state'])
+
+        # Now update status_message while in SKIPPED state
+        response = self.patch_json(
+            '/actions/%s' % self.action.uuid,
+            [{'path': '/status_message', 'value': 'updated message',
+              'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'})
+        self.assertEqual(HTTPStatus.OK, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(new_state, response.json['state'])
+        self.assertEqual(
+            'Action skipped by user. Reason: updated message',
+            response.json['status_message'])
 
 
 class TestActionPolicyEnforcement(api_base.FunctionalTest):
@@ -562,6 +751,16 @@ class TestActionPolicyEnforcement(api_base.FunctionalTest):
             '/actions/detail',
             expect_errors=True)
 
+    def test_policy_disallow_patch(self):
+        action = obj_utils.create_test_action(self.context)
+        self._common_policy_check(
+            "action:update", self.patch_json,
+            '/actions/%s' % action.uuid,
+            [{'path': '/state', 'value': objects.action.State.SKIPPED,
+              'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'infra-optim 1.5'},
+            expect_errors=True)
+
 
 class TestActionPolicyEnforcementWithAdminContext(TestListAction,
                                                   api_base.AdminRoleTest):
