@@ -10,9 +10,10 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+from http import HTTPStatus
 from oslo_config import cfg
 from oslo_serialization import jsonutils
-from six.moves.urllib import parse as urlparse
+from urllib import parse as urlparse
 
 from watcher.common import utils
 from watcher.tests.api import base as api_base
@@ -59,7 +60,7 @@ class TestListGoal(api_base.FunctionalTest):
         response = self.get_json(
             '/goals/%s' % goal['uuid'],
             expect_errors=True)
-        self.assertEqual(404, response.status_int)
+        self.assertEqual(HTTPStatus.NOT_FOUND, response.status_int)
 
     def test_detail(self):
         goal = obj_utils.create_test_goal(self.context)
@@ -71,7 +72,7 @@ class TestListGoal(api_base.FunctionalTest):
         goal = obj_utils.create_test_goal(self.context)
         response = self.get_json('/goals/%s/detail' % goal.uuid,
                                  expect_errors=True)
-        self.assertEqual(404, response.status_int)
+        self.assertEqual(HTTPStatus.NOT_FOUND, response.status_int)
 
     def test_many(self):
         goal_list = []
@@ -139,7 +140,7 @@ class TestListGoal(api_base.FunctionalTest):
         response = self.get_json(
             '/goals?sort_key=%s' % 'bad_name',
             expect_errors=True)
-        self.assertEqual(400, response.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, response.status_int)
 
 
 class TestGoalPolicyEnforcement(api_base.FunctionalTest):
@@ -150,7 +151,7 @@ class TestGoalPolicyEnforcement(api_base.FunctionalTest):
             "default": "rule:admin_api",
             rule: "rule:default"})
         response = func(*arg, **kwarg)
-        self.assertEqual(403, response.status_int)
+        self.assertEqual(HTTPStatus.FORBIDDEN, response.status_int)
         self.assertEqual('application/json', response.content_type)
         self.assertTrue(
             "Policy doesn't allow %s to be performed." % rule,
