@@ -79,6 +79,13 @@ class TestZoneMigration(TestBaseStrategy):
         self.m_priority = p_priority.start()
         self.addCleanup(p_priority.stop)
 
+        p_with_attached_volume = mock.patch.object(
+            strategies.ZoneMigration, "with_attached_volume",
+            new_callable=mock.PropertyMock
+        )
+        self.m_with_attached_volume = p_with_attached_volume.start()
+        self.addCleanup(p_with_attached_volume.stop)
+
         model = self.fake_c_cluster.generate_scenario_1()
         self.m_c_model.return_value = model
 
@@ -99,6 +106,7 @@ class TestZoneMigration(TestBaseStrategy):
             {"src_pool": "src2@back1#pool1", "dst_pool": "dst2@back2#pool1",
              "src_type": "type2", "dst_type": "type3"}
         ]
+        self.m_with_attached_volume.return_value = False
 
         self.strategy = strategies.ZoneMigration(
             config=mock.Mock())
@@ -154,6 +162,7 @@ class TestZoneMigration(TestBaseStrategy):
         setattr(volume, 'size', kwargs.get('size', '1'))
         setattr(volume, 'created_at',
                 kwargs.get('created_at', '1977-01-01T00:00:00'))
+        setattr(volume, 'attachments', kwargs.get('attachments', []))
         volume.volume_type = kwargs.get('volume_type', 'type1')
 
         return volume
@@ -245,6 +254,22 @@ class TestZoneMigration(TestBaseStrategy):
         global_efficacy_value = solution.global_efficacy[0].get('value', 0)
         self.assertEqual(100, global_efficacy_value)
 
+    def test_execute_live_migrate_instance_no_dst_node(self):
+        instance_on_src1 = self.fake_instance(
+            host="src1",
+            id="INSTANCE_1",
+            name="INSTANCE_1")
+        self.m_n_helper.get_instance_list.return_value = [
+            instance_on_src1,
+        ]
+        self.m_c_helper.get_volume_list.return_value = []
+        self.m_migrate_compute_nodes.return_value = [{"src_node": "src1"}]
+        solution = self.strategy.execute()
+        migration_params = solution.actions[0]['input_parameters']
+        # since we have not passed 'dst_node' in the input, we should not have
+        # a destination_node in the generated migration action
+        self.assertNotIn('destination_node', migration_params)
+
     def test_execute_cold_migrate_instance(self):
         instance_on_src1 = self.fake_instance(
             host="src1",
@@ -285,6 +310,84 @@ class TestZoneMigration(TestBaseStrategy):
         global_efficacy_value = solution.global_efficacy[2].get('value', 0)
         self.assertEqual(100, global_efficacy_value)
 
+    def test_execute_migrate_volume_no_dst_pool(self):
+        volume_on_src1 = self.fake_volume(host="src1@back1#pool1",
+                                          id=volume_uuid_mapping["volume_1"],
+                                          name="volume_1")
+        self.m_c_helper.get_volume_list.return_value = [
+            volume_on_src1,
+
+            ]
+        self.m_migrate_storage_pools.return_value = [
+            {"src_pool": "src1@back1#pool1",
+             "src_type": "type1",
+             "dst_type": "type1"},
+
+            ]
+        self.m_n_helper.get_instance_list.return_value = []
+        solution = self.strategy.execute()
+        # check that there are no volume migrations proposed, because the input
+        # parameters do not have a dst_pool and dst_type==src_type
+        self.assertEqual(len(solution.actions), 0)
+
+    def test_execute_migrate_volume_dst_pool(self):
+        volume_on_src1 = self.fake_volume(host="src1@back1#pool1",
+                                          id=volume_uuid_mapping["volume_1"],
+                                          name="volume_1")
+        self.m_c_helper.get_volume_list.return_value = [
+            volume_on_src1,
+
+            ]
+        self.m_migrate_storage_pools.return_value = [
+            {"src_pool": "src1@back1#pool1",
+             "src_type": "type1",
+             "dst_pool": "back2",
+             "dst_type": "type1"},
+
+            ]
+        self.m_n_helper.get_instance_list.return_value = []
+        solution = self.strategy.execute()
+        # check that there is one volume migration proposed
+        migration_types = collections.Counter(
+            [action.get('input_parameters')['migration_type']
+             for action in solution.actions])
+        self.assertEqual(1, migration_types.get("migrate", 0))
+        global_efficacy_value = solution.global_efficacy[2].get('value', 0)
+        self.assertEqual(100, global_efficacy_value)
+
+    def test_execute_migrate_volume_no_compute_nodes(self):
+        instance_on_src1 = self.fake_instance(
+            host="src1",
+            id="INSTANCE_1",
+            name="INSTANCE_1")
+        vol_attach = [{"server_id": instance_on_src1.id}]
+        volume_on_src1 = self.fake_volume(host="src1@back1#pool1",
+                                          id=volume_uuid_mapping["volume_1"],
+                                          name="volume_1",
+                                          status="in-use",
+                                          attachments=vol_attach)
+        self.m_c_helper.get_volume_list.return_value = [
+            volume_on_src1,
+            ]
+        self.m_migrate_storage_pools.return_value = [
+            {"src_pool": "src1@back1#pool1",
+             "dst_pool": "dst1@back1#pool1",
+             "src_type": "type1", "dst_type": "type1"},
+            ]
+        self.m_migrate_compute_nodes.return_value = None
+        self.m_n_helper.get_instance_list.return_value = [
+            instance_on_src1,
+        ]
+        self.m_with_attached_volume.return_value = True
+
+        # check that the solution contains one 'swap' volume migration and one
+        # instance migration, once the bug is fixed
+        # solution = self.strategy.execute()
+
+        # temporarily catch the error to demonstrate it
+        exception = self.assertRaises(TypeError, self.strategy.execute)
+        self.assertEqual(str(exception), "'NoneType' object is not iterable")
+
     def test_execute_retype_volume(self):
         volume_on_src2 = self.fake_volume(host="src2@back1#pool1",
                                           id=volume_uuid_mapping["volume_2"],
@@ -320,7 +423,10 @@ class TestZoneMigration(TestBaseStrategy):
         migration_types = collections.Counter(
             [action.get('input_parameters')['migration_type']
              for action in solution.actions])
-        self.assertEqual(1, migration_types.get("swap", 0))
+        # watcher no longer implements swap. it is now an
+        # alias for migrate.
+        self.assertEqual(0, migration_types.get("swap", 0))
+        self.assertEqual(1, migration_types.get("migrate", 1))
         global_efficacy_value = solution.global_efficacy[3].get('value', 0)
         self.assertEqual(100, global_efficacy_value)
 
